@@ -30,16 +30,21 @@
 #'
 #' **The algorithm**
 #'
-#' This implementation of DBSCAN (Hahsler et al, 2019) follows the original
-#' algorithm as described by Ester et al (1996). DBSCAN estimates the density
-#' around each data point by counting the number of points in a user-specified
-#' eps-neighborhood and applies a used-specified minPts thresholds to identify
-#' core, border and noise points. In a second step, core points are joined into
-#' a cluster if they are density-reachable (i.e., there is a chain of core
-#' points where one falls inside the eps-neighborhood of the next). Finally,
-#' border points are assigned to clusters. The algorithm needs parameters
-#' `eps` (the radius of the epsilon neighborhood) and `minPts` (the
-#' density threshold).
+#' This implementation of DBSCAN follows the original
+#' algorithm as described by Ester et al (1996). DBSCAN performs the following steps:
+#'
+#' 1. Estimate the density
+#'   around each data point by counting the number of points in a user-specified
+#'   eps-neighborhood and applies a used-specified minPts thresholds to identify
+#'      - core points (points with more than minPts points in their neighborhood),
+#'      - border points (non-core points with a core point in their neighborhood) and
+#'      - noise points (all other points).
+#' 2. Core points form the backbone of clusters by joining them into
+#'   a cluster if they are density-reachable from each other (i.e., there is a chain of core
+#'   points where one falls inside the eps-neighborhood of the next).
+#' 3. Border points are assigned to clusters. The algorithm needs parameters
+#'   `eps` (the radius of the epsilon neighborhood) and `minPts` (the
+#'   density threshold).
 #'
 #' Border points are arbitrarily assigned to clusters in the original
 #' algorithm. DBSCAN* (see Campello et al 2013) treats all border points as
@@ -50,7 +55,7 @@
 #' If `x` is a matrix or a data.frame, then fast fixed-radius nearest
 #' neighbor computation using a kd-tree is performed using Euclidean distance.
 #' See [frNN()] for more information on the parameters related to
-#' nearest neighbor search.
+#' nearest neighbor search. **Note** that only numerical values are allowed in `x`.
 #'
 #' Any precomputed distance matrix (dist object) can be specified as `x`.
 #' You may run into memory issues since distance matrices are large.
@@ -62,16 +67,28 @@
 #'
 #' **Setting parameters for DBSCAN**
 #'
-#' The parameters `minPts` and `eps` depend on each other and
-#' changing one typically requires changing the other one as well. The original
-#' DBSCAN paper suggests to start by setting `minPts` to the
-#' dimensionality of the data plus one or higher. `minPts` defines the
-#' minimum density around a core point (i.e., the minimum density for non-noise
-#' areas). Increase the parameter to suppress more noise in the data and
-#' require more points to form a cluster. A suitable neighborhood size
+#' The parameters `minPts` and `eps` define the minimum density required
+#' in the area around core points which form the backbone of clusters.
+#' `minPts` is the number of points
+#' required in the neighborhood around the point defined by the parameter `eps`
+#' (i.e., the radius around the point). Both parameters
+#' depend on each other and changing one typically requires changing
+#' the other one as well. The parameters also depend on the size of the data set with
+#' larger datasets requiring a larger `minPts` or a smaller `eps`.
+#'
+#' * `minPts:` The original
+#' DBSCAN paper (Ester et al, 1996) suggests to start by setting \eqn{\text{minPts} \ge d + 1},
+#' the data dimensionality plus one or higher with a minimum of 3. Larger values
+#' are preferable since increasing the parameter suppresses more noise in the data
+#' by requiring more points to form clusters.
+#' Sander et al (1998) uses in the examples two times the data dimensionality.
+#' Note that setting \eqn{\text{minPts} \le 2} is equivalent to hierarchical clustering
+#' with the single link metric and the dendrogram cut at height `eps`.
+#'
+#' * `eps:` A suitable neighborhood size
 #' parameter `eps` given a fixed value for `minPts` can be found
 #' visually by inspecting the [kNNdistplot()] of the data using
-#' `k = minPts - 1` (`minPts` includes the point itself, while the
+#' \eqn{k = \text{minPts} - 1} (`minPts` includes the point itself, while the
 #' k-nearest neighbors distance does not). The k-nearest neighbor distance plot
 #' sorts all data points by their k-nearest neighbor distance. A sudden
 #' increase of the kNN distance (a knee) indicates that the points to the right
@@ -81,9 +98,16 @@
 #'
 #' [predict()] can be used to predict cluster memberships for new data
 #' points. A point is considered a member of a cluster if it is within the eps
-#' neighborhood of a member of the cluster (Euclidean distance is used). Points
-#' which cannot be assigned to a cluster will be reported as members of the
-#' noise cluster 0.
+#' neighborhood of a core point of the cluster. Points
+#' which cannot be assigned to a cluster will be reported as
+#' noise points (i.e., cluster ID 0).
+#' **Important note:** `predict()` currently can only use Euclidean distance to determine
+#' the neighborhood of core points. If `dbscan()` was called using distances other than Euclidean,
+#' then the neighborhood calculation will not be correct and only approximated by Euclidean
+#' distances. If the data contain factor columns (e.g., using Gower's distance), then
+#' the factors in `data` and `query` first need to be converted to numeric to use the
+#' Euclidean approximation.
+#'
 #'
 #' @aliases dbscan DBSCAN print.dbscan_fast
 #' @family clustering functions
@@ -103,12 +127,16 @@
 #' neighbor search algorithm. See [frNN()] for details on how to
 #' control the search strategy.
 #'
-#' @return An object of class `dbscan_fast` with the following components:
+#' @return `dbscan()` returns an object of class `dbscan_fast` with the following components:
 #'
 #' \item{eps }{ value of the `eps` parameter.}
 #' \item{minPts }{ value of the `minPts` parameter.}
+#' \item{metric }{ used distance metric.}
 #' \item{cluster }{A integer vector with cluster assignments. Zero indicates noise points.}
 #'
+#' `is.corepoint()` returns a logical vector indicating for each data point if it is a
+#'   core point.
+#'
 #' @author Michael Hahsler
 #' @references Hahsler M, Piekenbrock M, Doran D (2019). dbscan: Fast
 #' Density-Based Clustering with R.  _Journal of Statistical Software,_
@@ -127,6 +155,12 @@
 #' 17th Pacific-Asia Conference on Knowledge Discovery in Databases, PAKDD
 #' 2013, _Lecture Notes in Computer Science_ 7819, p. 160.
 #' \doi{10.1007/978-3-642-37456-2_14}
+#'
+#' Sander, J., Ester, M., Kriegel, HP. et al. (1998). Density-Based
+#' Clustering in Spatial Databases: The Algorithm GDBSCAN and Its Applications.
+#' _Data Mining and Knowledge Discovery_ 2, 169-194.
+#' \doi{10.1023/A:1009745219419}
+#'
 #' @keywords model clustering
 #' @examples
 #' ## Example 1: use dbscan on the iris data set
@@ -136,7 +170,7 @@
 #' ## Find suitable DBSCAN parameters:
 #' ## 1. We use minPts = dim + 1 = 5 for iris. A larger value can also be used.
 #' ## 2. We inspect the k-NN distance plot for k = minPts - 1 = 4
-#' kNNdistplot(iris, k = 5 - 1)
+#' kNNdistplot(iris, minPts = 5)
 #'
 #' ## Noise seems to start around a 4-NN distance of .7
 #' abline(h=.7, col = "red", lty = 2)
@@ -211,7 +245,7 @@
 #' nn
 #' dbscan(nn, minPts = 2)
 #'
-#' @export dbscan
+#' @export
 dbscan <-
   function(x,
     eps,
@@ -222,16 +256,26 @@ dbscan <-
     if (inherits(x, "frNN") && missing(eps))
       eps <- x$eps
 
+    if (inherits(x, "dist")) {
+      .check_dist(x)
+      dist_method <- attr(x, "method")
+    } else
+      dist_method <- "euclidean"
+
+    if (is.null(dist_method))
+      dist_method <- "unknown"
+
     ### extra contains settings for frNN
     ### search = "kdtree", bucketSize = 10, splitRule = "suggest", approx = 0
-    ### also check for MinPts for fpc compartibility (does not work for
+    ### also check for MinPts for fpc compatibility (does not work for
     ### search method dist)
     extra <- list(...)
-    args <- c("MinPts", "search", "bucketSize", "splitRule", "approx")
+    args <-
+      c("MinPts", "search", "bucketSize", "splitRule", "approx")
     m <- pmatch(names(extra), args)
-    if (any(is.na(m)))
+    if (anyNA(m))
       stop("Unknown parameter: ",
-        paste(names(extra)[is.na(m)], collapse = ", "))
+        toString(names(extra)[is.na(m)]))
     names(extra) <- args[m]
 
     if (!is.null(extra$MinPts)) {
@@ -289,16 +333,16 @@ dbscan <-
 
     } else{
       if (!.matrixlike(x))
-        stop("x needs to be a matrix")
+        stop("x needs to be a matrix or data.frame.")
       ## make sure x is numeric
       x <- as.matrix(x)
       if (storage.mode(x) == "integer")
         storage.mode(x) <- "double"
       if (storage.mode(x) != "double")
-        stop("x has to be a numeric matrix.")
+        stop("all data in x has to be numeric.")
     }
 
-    if (length(frNN) == 0 && any(is.na(x)))
+    if (length(frNN) == 0 && anyNA(x))
       stop("data/distances cannot contain NAs for dbscan (with kd-tree)!")
 
     ## add self match and use C numbering if frNN is used
@@ -332,15 +376,19 @@ dbscan <-
       frNN
     )
 
-    structure(list(
-      cluster = ret,
-      eps = eps,
-      minPts = minPts
-    ),
-      class = c("dbscan_fast", "dbscan"))
+    structure(
+      list(
+        cluster = ret,
+        eps = eps,
+        minPts = minPts,
+        metric = dist_method,
+        borderPoints = borderPoints
+      ),
+      class = c("dbscan_fast", "dbscan")
+    )
   }
 
-
+#' @export
 print.dbscan_fast <- function(x, ...) {
   cl <- unique(x$cluster)
   cl <- length(cl[cl != 0L])
@@ -348,6 +396,12 @@ print.dbscan_fast <- function(x, ...) {
   writeLines(c(
     paste0("DBSCAN clustering for ", length(x$cluster), " objects."),
     paste0("Parameters: eps = ", x$eps, ", minPts = ", x$minPts),
+    paste0(
+      "Using ",
+      x$metric,
+      " distances and borderpoints = ",
+      x$borderPoints
+    ),
     paste0(
       "The clustering contains ",
       cl,
@@ -362,6 +416,11 @@ print.dbscan_fast <- function(x, ...) {
 
   writeLines(strwrap(paste0(
     "Available fields: ",
-    paste(names(x), collapse = ", ")
+    toString(names(x))
   ), exdent = 18))
 }
+
+#' @rdname dbscan
+#' @export
+is.corepoint <- function(x, eps, minPts = 5, ...)
+  lengths(frNN(x, eps = eps, ...)$id) >= (minPts - 1)
