@@ -22,17 +22,24 @@
 
 #' Find Shared Nearest Neighbors
 #'
-#' Calculates the number of shared nearest neighbors, the shared nearest
-#' neighbor similarity and creates a shared nearest neighbors graph.
+#' Calculates the number of shared nearest neighbors
+#' and creates a shared nearest neighbors graph.
 #'
-#' The number of shared nearest neighbors is the intersection of the kNN
-#' neighborhood of two points. Note: that each point is considered to be part
-#' of its own kNN neighborhood. The range for the shared nearest neighbors is
-#' \eqn{[0, k]}.
+#' The number of shared nearest neighbors of two points p and q is the
+#' intersection of the kNN neighborhood of two points.
+#' Note: that each point is considered to be part
+#' of its own kNN neighborhood.
+#' The range for the shared nearest neighbors is
+#' \eqn{[0, k]}. The result is a n-by-k matrix called `shared`.
+#' Each row is a point and the columns are the point's k nearest neighbors.
+#' The value is the count of the shared neighbors.
 #'
-#' Javis and Patrick (1973) use the shared nearest neighbor graph for
-#' clustering. They only count shared neighbors between points that are in each
-#' other's kNN neighborhood.
+#' The shared nearest neighbor graph connects a point with all its nearest neighbors
+#' if they have at least one shared neighbor. The number of shared neighbors can be used
+#' as an edge weight.
+#' Javis and Patrick (1973) use a slightly
+#' modified (see parameter `jp`) shared nearest neighbor graph for
+#' clustering.
 #'
 #' @aliases sNN snn
 #' @family NN functions
@@ -43,10 +50,10 @@
 #' @param kt minimum threshold on the number of shared nearest neighbors to
 #' build the shared nearest neighbor graph. Edges are only preserved if
 #' `kt` or more neighbors are shared.
-#' @param jp use the definition by Javis and Patrick (1973), where shared
-#' neighbors are only counted between points that are in each other's
-#' neighborhood, otherwise 0 is returned. If `FALSE`, then the number of shared
-#' neighbors is returned, even if the points are not neighbors.
+#' @param jp In regular sNN graphs, two points that are not neighbors
+#' can have shared neighbors.
+#' Javis and Patrick (1973) requires the two points to be neighbors, otherwise
+#' the count is zeroed out. `TRUE` uses this behavior.
 #' @param search nearest neighbor search strategy (one of `"kdtree"`, `"linear"` or
 #' `"dist"`).
 #' @param sort sort by the number of shared nearest neighbors? Note that this
@@ -60,12 +67,15 @@
 #' a factor of `(1 + approx) eps` may be used. Some actual NN may be omitted
 #' leading to spurious clusters and noise points.  However, the algorithm will
 #' enjoy a significant speedup.
+#' @param decreasing logical; sort in decreasing order?
+#' @param ... additional parameters are passed on.
 #' @return An object of class `sNN` (subclass of [kNN] and [NN]) containing a list
 #' with the following components:
 #' \item{id }{a matrix with ids. }
 #' \item{dist}{a matrix with the distances. }
 #' \item{shared }{a matrix with the number of shared nearest neighbors. }
 #' \item{k }{number of `k` used. }
+#' \item{metric }{the used distance metric. }
 #'
 #' @author Michael Hahsler
 #' @references R. A. Jarvis and E. A. Patrick. 1973. Clustering Using a
@@ -85,7 +95,7 @@
 #' # shared nearest neighbor distribution
 #' table(as.vector(nn$shared))
 #'
-#' # explore neighborhood of point 10
+#' # explore number of shared points for the k-neighborhood of point 10
 #' i <- 10
 #' nn$shared[i,]
 #'
@@ -98,7 +108,7 @@
 #'
 #' # get an adjacency list for the shared nearest neighbor graph
 #' adjacencylist(nn_3)
-#' @export sNN
+#' @export
 sNN <- function(x,
   k,
   kt = NULL,
@@ -156,9 +166,10 @@ sNN <- function(x,
   x
 }
 
-
+#' @rdname sNN
+#' @export
 sort.sNN <- function(x, decreasing = TRUE, ...) {
-  if (!is.null(x$sort_shared) && x$sort_shared)
+  if (isTRUE(x$sort_shared))
     return(x)
   if (is.null(x$shared))
     stop("Unable to sort. Number of shared neighbors is missing.")
@@ -170,11 +181,10 @@ sort.sNN <- function(x, decreasing = TRUE, ...) {
 
   ## sort first by number of shared points (decreasing) and break ties by id (increasing)
   k <- ncol(x$shared)
-  o <- sapply(
+  o <- vapply(
     1:nrow(x$shared),
-    FUN =
-      function(i)
-        order(k - x$shared[i, ], x$id[i, ], decreasing = !decreasing)
+    function(i) order(k - x$shared[i, ], x$id[i, ], decreasing = !decreasing),
+    integer(k)
   )
   for (i in 1:ncol(o)) {
     x$shared[i, ] <- x$shared[i, ][o[, i]]
@@ -188,6 +198,8 @@ sort.sNN <- function(x, decreasing = TRUE, ...) {
   x
 }
 
+#' @rdname sNN
+#' @export
 print.sNN <- function(x, ...) {
   cat(
     "shared-nearest neighbors for ",
@@ -200,5 +212,5 @@ print.sNN <- function(x, ...) {
     "\n",
     sep = ""
   )
-  cat("Available fields: ", paste(names(x), collapse = ", "), "\n", sep = "")
+  cat("Available fields: ", toString(names(x)), "\n", sep = "")
 }
